package com.zapylaev.webchat.command.request;

import com.google.gson.Gson;
import com.zapylaev.webchat.command.response.*;
import com.zapylaev.webchat.properties.Pages;
import com.zapylaev.webchat.message.Message;
import com.zapylaev.webchat.message.MessageStore;
import org.apache.log4j.Logger;

import javax.servlet.AsyncContext;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * User: Kirill.Zapylaev
 * Date: 14.03.13
 * Time: 0:31
 */
public class ChatCommand implements Command {

    public static final Logger LOG =Logger.getLogger(ChatCommand.class);

    private final String GET_REQUEST = "GET";

    private List<AsyncContext> contexts = new LinkedList<AsyncContext>();

    @Override
    public CommandResponse execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        if (request.getMethod().equals(GET_REQUEST)) {
            AsyncContext asyncContext = request.startAsync(request, response);
            LOG.info("GET calls, create asyncContext = " + asyncContext);
            asyncContext.setTimeout(10 * 60 * 1000);
            contexts.add(asyncContext);

            return new EmptyResponse();
        }

        List<AsyncContext> asyncContexts = new ArrayList<AsyncContext>(contexts);
        contexts.clear();

        String messageText = request.getParameter("message");
        LOG.info("Message test is : " + messageText);

        Message message = new Message();
        SimpleDateFormat dateFormat = new SimpleDateFormat(request.getServletContext().getInitParameter("dateFormat"));
        message.setDate(dateFormat.format(new Date()));
        message.setMessage(messageText);
        message.setUserName(request.getServletContext().getInitParameter("defaultUsername"));
        MessageStore.getInstance().storeMessage(message);

        List<Message> messages = MessageStore.getInstance().getMessages();

        Gson gson = new Gson();
        String jsonMessages = gson.toJson(messages);

        LOG.info(jsonMessages);

        for (AsyncContext asyncContext : asyncContexts) {
            LOG.info("asyncContext is : " + asyncContext);
            LOG.info("set messages: " + messages);

            // TODO make filter
            asyncContext.getResponse().setCharacterEncoding("utf-8");

            PrintWriter writer = asyncContext.getResponse().getWriter();
            writer.print(jsonMessages);
            writer.flush();
            asyncContext.complete();
        }

        return new RedirectResponse(Pages.getPagePath(Pages.CHAT_PAGE));
    }
}
