package com.zapylaev.webchat.command.request;

import com.zapylaev.webchat.bean.User;
import com.zapylaev.webchat.command.helper.AttributesHelper;
import com.zapylaev.webchat.command.request.validator.InputValidator;
import com.zapylaev.webchat.command.response.CommandResponse;
import com.zapylaev.webchat.command.response.RedirectResponse;
import com.zapylaev.webchat.persistence.dao.DAOFactory;
import com.zapylaev.webchat.properties.Pages;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * User: Kirill.Zapylaev
 * Date: 17.03.13
 * Time: 0:02
 */
public class RegistrationCommand implements Command {
    @Override
    public CommandResponse execute(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String login = request.getParameter(AttributesHelper.LOGIN);
        String password = request.getParameter(AttributesHelper.PASSWORD);
        String passwordRetype = request.getParameter(AttributesHelper.PASSWORD_RETYPE);

        int result = InputValidator.validateRegistration(password, passwordRetype, login);

        switch (result) {
            case InputValidator.SUCCESS:
                return doRegistration(login, password, request);

            case InputValidator.PASSWORD_EMPTY:
                request.getServletContext().setAttribute(AttributesHelper.REGISTRATION_FAILED, "Password should not be empty");
                break;

            case InputValidator.PASSWORDS_DO_NOT_MATCH:
                request.getServletContext().setAttribute(AttributesHelper.REGISTRATION_FAILED, "Passwords do not match");
                break;

            case InputValidator.LOGIN_IS_EXIST:
                request.getServletContext().setAttribute(AttributesHelper.REGISTRATION_FAILED, "This login is already used");
                break;
        }

        return new RedirectResponse(Pages.getPagePath(Pages.REGISTRATION_PAGE));
    }

    private CommandResponse doRegistration(String login, String password, HttpServletRequest request) {
        User user = new User();
        user.setLogin(login);
        user.setPassword(password);
        DAOFactory.createUserDAO().addUser(user);

        request.getServletContext().setAttribute(AttributesHelper.LOGIN, login);
        return new RedirectResponse(Pages.getPagePath(Pages.REGISTRATION_SUCCESS_PAGE));
    }
}
