package com.zapylaev.webchat.persistence.connection;

import com.zapylaev.webchat.exception.DAOException;
import org.apache.log4j.Logger;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

/**
 * @author Kirill Zapylaiev <zapylaev@gmail.com>
 */
public class ChatConnection {
    private static final Logger LOGGER = Logger.getLogger(ChatConnection.class);
    private static final String SQL_PROPERTIES_NAME = "/mysql.properties";
    private static final String URL = "URL";
    private static final String DRIVER_NAME = "DRIVER_NAME";

    private Properties connectionProperties = new Properties();

    private static ChatConnection instance;

    public static synchronized ChatConnection getInstance() {
        if (instance == null) {
            instance = new ChatConnection();
        }
        return instance;
    }

    private ChatConnection() {
        InputStream propertiesFile;
        try {
            propertiesFile = ChatConnection.class.getResourceAsStream(SQL_PROPERTIES_NAME);

            if (propertiesFile == null) {
                throw new DAOException("Unable to load sql properties file");
            }

            connectionProperties.load(propertiesFile);
        } catch (FileNotFoundException e) {
            throw new DAOException(e.getMessage(), e);
        } catch (IOException e) {
            throw new DAOException(e.getMessage(), e);
        }
    }

    public Connection getConnection() {
        try {
            Class.forName(connectionProperties.getProperty(DRIVER_NAME));
        }
        catch(Exception e) {
            LOGGER.error("Unable to load jdbc driver", e);
            throw new DAOException(e.getMessage(), e);
        }

        Connection connection;
        try {
            connection = DriverManager.getConnection(connectionProperties.getProperty(URL), connectionProperties);
        }
        catch (SQLException e) {
            throw new DAOException(e.getMessage(), e);
        }

        return connection;
    }

    public void closeConnection(Connection connection) {
        try {
            connection.close();
        } catch (SQLException e) {
            LOGGER.error(e.getMessage(), e);
        }
    }
}
